#include "FileSystem.h"
#include "CS240Exception.h"
#include "UnitTest.h"

#include <sstream>
#include <iostream>
using namespace std;


bool FileSystem::FileExists(const string & path) {

	struct stat info;
	return (ReadFileInfo(path, &info) && S_ISREG(info.st_mode));
}


bool FileSystem::DirectoryExists(const string & path) {

	struct stat info;	
	return (ReadFileInfo(path, &info) && (info.st_mode & S_IFDIR));
}


int FileSystem::CompareFileTimestamps(const string & path1, const string & path2) {
	
	struct stat info1;
	struct stat info2;
	
	if (!ReadFileInfo(path1, &info1)) {
		throw CS240Exception(FileDoesNotExistErrorMessage(path1));
	}
	
	if (!ReadFileInfo(path2, &info2)) {
		throw CS240Exception(FileDoesNotExistErrorMessage(path2));
	}
	
	if (info1.st_mtime < info2.st_mtime) {
		return -1;
	}
	else if (info1.st_mtime > info2.st_mtime) {
		return 1;
	}
	else {
		return 0;
	}
}


bool FileSystem::ReadFileInfo(const string & path, struct stat * info) {
	
	return (::stat(path.c_str(), info) == 0);
}


string FileSystem::FileDoesNotExistErrorMessage(const string & path) {
	ostringstream message;
	message << "file does not exist: " << path;
	return message.str();	
}


bool FileSystem::Test(ostream & os) {
	
	bool success = true;
	
	TEST(FileSystem::FileExists("test/oldfile1.txt"));
	TEST(FileSystem::FileExists("test/newfile1.txt"));
	TEST(!FileSystem::FileExists("test/bogusfile.txt"));
	TEST(!FileSystem::FileExists("test"));
	
	TEST(FileSystem::DirectoryExists("test/testdir"));
	TEST(!FileSystem::DirectoryExists("test/bogusdir"));
	TEST(!FileSystem::DirectoryExists("test/oldfile1.txt"));
	
	TEST(FileSystem::CompareFileTimestamps("test/oldfile1.txt", "test/oldfile1.txt") == 0);
	TEST(FileSystem::CompareFileTimestamps("test/newfile1.txt", "test/newfile1.txt") == 0);
	TEST(FileSystem::CompareFileTimestamps("test/oldfile1.txt", "test/newfile1.txt") < 0);
	TEST(FileSystem::CompareFileTimestamps("test/newfile1.txt", "test/oldfile1.txt") > 0);

	return success;
}
